@@ -13,7 +13,7 @@ from trytond.modules.account_product.product import (
     account_used, template_property)
 from trytond.modules.product import price_digits
 
-__all__ = ['Category', 'CategoryAccount', 'Template', 'TemplateAccount',
+__all__ = ['Category', 'CategoryAccount', 'Template',
     'Product', 'UpdateCostPriceAsk',
     'UpdateCostPriceShowMove', 'UpdateCostPrice']
 account_names = [
@@ -21,8 +21,7 @@ account_names = [
     'account_stock_production', 'account_stock_lost_found']
 
 
-class Category:
-    __metaclass__ = PoolMeta
+class Category(metaclass=PoolMeta):
     __name__ = 'product.category'
     account_stock = fields.MultiValue(fields.Many2One(
             'account.account', "Account Stock",
@@ -118,8 +117,7 @@ class Category:
         pass
 
 
-class CategoryAccount:
-    __metaclass__ = PoolMeta
+class CategoryAccount(metaclass=PoolMeta):
     __name__ = 'product.category.account'
     account_stock = fields.Many2One(
         'account.account', "Account Stock",
@@ -162,7 +160,7 @@ class CategoryAccount:
         TableHandler = backend.get('TableHandler')
         exist = TableHandler.table_exist(cls._table)
         if exist:
-            table = TableHandler(cls, module_name)
+            table = cls.__table_handler__(module_name)
             exist &= all(table.column_exist(c) for c in account_names)
 
         super(CategoryAccount, cls).__register__(module_name)
@@ -179,74 +177,8 @@ class CategoryAccount:
             field_names, value_names, fields)
 
 
-class Template:
-    __metaclass__ = PoolMeta
+class Template(metaclass=PoolMeta):
     __name__ = 'product.template'
-    account_stock = fields.MultiValue(fields.Many2One(
-            'account.account', "Account Stock",
-            domain=[
-                ('kind', '=', 'stock'),
-                ('company', '=', Eval('context', {}).get('company', -1)),
-                ],
-            states={
-                'invisible': (~Eval('context', {}).get('company')
-                    | Eval('account_category')
-                    | (Eval('type') != 'goods')),
-                }, help='This account will be used instead of the one defined '
-            'on the category.',
-            depends=['account_category', 'type']))
-    account_stock_supplier = fields.MultiValue(fields.Many2One(
-            'account.account', "Account Stock Supplier",
-            domain=[
-                ('kind', '=', 'stock'),
-                ('company', '=', Eval('context', {}).get('company', -1)),
-                ],
-            states={
-                'invisible': (~Eval('context', {}).get('company')
-                    | Eval('account_category')
-                    | (Eval('type') != 'goods')),
-                }, help='This account will be used instead of the one defined '
-            'on the category.',
-            depends=['account_category', 'type']))
-    account_stock_customer = fields.MultiValue(fields.Many2One(
-            'account.account', "Account Stock Customer",
-            domain=[
-                ('kind', '=', 'stock'),
-                ('company', '=', Eval('context', {}).get('company', -1)),
-                ],
-            states={
-                'invisible': (~Eval('context', {}).get('company')
-                    | Eval('account_category')
-                    | (Eval('type') != 'goods')),
-                }, help='This account will be used instead of the one defined '
-            'on the category.',
-            depends=['account_category', 'type']))
-    account_stock_production = fields.MultiValue(fields.Many2One(
-            'account.account', "Account Stock Production",
-            domain=[
-                ('kind', '=', 'stock'),
-                ('company', '=', Eval('context', {}).get('company', -1)),
-                ],
-            states={
-                'invisible': (~Eval('context', {}).get('company')
-                    | Eval('account_category')
-                    | (Eval('type') != 'goods')),
-                }, help='This account will be used instead of the one defined '
-            'on the category.',
-            depends=['account_category', 'type']))
-    account_stock_lost_found = fields.MultiValue(fields.Many2One(
-            'account.account', 'Account Stock Lost and Found',
-            domain=[
-                ('kind', '=', 'stock'),
-                ('company', '=', Eval('context', {}).get('company', -1)),
-                ],
-            states={
-                'invisible': (~Eval('context', {}).get('company')
-                    | Eval('account_category')
-                    | (Eval('type') != 'goods')),
-                }, help='This account will be used instead of the one defined '
-            'on the category.',
-            depends=['account_category', 'type']))
 
     @classmethod
     def __setup__(cls):
@@ -258,102 +190,33 @@ class Template:
                     'You must use the "Update Cost Price" wizard.'),
                 })
 
-    @classmethod
-    def multivalue_model(cls, field):
-        pool = Pool()
-        if field in account_names:
-            return pool.get('product.template.account')
-        return super(Template, cls).multivalue_model(field)
-
     @property
-    @account_used('account_stock')
+    @account_used('account_stock', 'account_category')
     def account_stock_used(self):
         pass
 
     @property
-    @account_used('account_stock_supplier')
+    @account_used('account_stock_supplier', 'account_category')
     def account_stock_supplier_used(self):
         pass
 
     @property
-    @account_used('account_stock_customer')
+    @account_used('account_stock_customer', 'account_category')
     def account_stock_customer_used(self):
         pass
 
     @property
-    @account_used('account_stock_production')
+    @account_used('account_stock_production', 'account_category')
     def account_stock_production_used(self):
         pass
 
     @property
-    @account_used('account_stock_lost_found')
+    @account_used('account_stock_lost_found', 'account_category')
     def account_stock_lost_found_used(self):
         pass
 
 
-class TemplateAccount:
-    __metaclass__ = PoolMeta
-    __name__ = 'product.template.account'
-    account_stock = fields.Many2One(
-        'account.account', "Account Stock",
-        domain=[
-            ('kind', '=', 'stock'),
-            ('company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
-    account_stock_supplier = fields.Many2One(
-        'account.account', "Account Stock Supplier",
-        domain=[
-            ('kind', '=', 'stock'),
-            ('company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
-    account_stock_customer = fields.Many2One(
-        'account.account', "Account Stock Customer",
-        domain=[
-            ('kind', '=', 'stock'),
-            ('company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
-    account_stock_production = fields.Many2One(
-        'account.account', "Account Stock Production",
-        domain=[
-            ('kind', '=', 'stock'),
-            ('company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
-    account_stock_lost_found = fields.Many2One(
-        'account.account', "Account Stock Lost and Found",
-        domain=[
-            ('kind', '=', 'stock'),
-            ('company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
-
-    @classmethod
-    def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        exist = TableHandler.table_exist(cls._table)
-        if exist:
-            table = TableHandler(cls, module_name)
-            exist &= all(table.column_exist(c) for c in account_names)
-
-        super(TemplateAccount, cls).__register__(module_name)
-
-        if not exist:
-            # Re-migration
-            cls._migrate_property([], [], [])
-
-    @classmethod
-    def _migrate_property(cls, field_names, value_names, fields):
-        field_names.extend(account_names)
-        value_names.extend(account_names)
-        super(TemplateAccount, cls)._migrate_property(
-            field_names, value_names, fields)
-
-
-class Product:
-    __metaclass__ = PoolMeta
+class Product(metaclass=PoolMeta):
     __name__ = 'product.product'
     account_stock_used = template_property('account_stock_used')
     account_stock_supplier_used = template_property(
@@ -409,7 +272,7 @@ class UpdateCostPrice(Wizard):
     ask_price = StateView('product.update_cost_price.ask',
         'account_stock_continental.update_cost_price_ask_form', [
             Button('Cancel', 'end', 'tryton-cancel'),
-            Button('OK', 'should_show_move', 'tryton-go-next', default=True),
+            Button('OK', 'should_show_move', 'tryton-forward', default=True),
             ])
     should_show_move = StateTransition()
     show_move = StateView('product.update_cost_price.show_move',
