@@ -9,8 +9,7 @@ from trytond.transaction import Transaction
 __all__ = ['Move']
 
 
-class Move:
-    __metaclass__ = PoolMeta
+class Move(metaclass=PoolMeta):
     __name__ = 'stock.move'
 
     def _get_account_stock_move_lines(self, type_):
@@ -118,18 +117,19 @@ class Move:
         type_ = self._get_account_stock_move_type()
         if not type_:
             return
-        if type_ == 'supplier_customer':
-            account_move_lines = self._get_account_stock_move_lines(
-                'in_supplier')
-            account_move_lines.extend(self._get_account_stock_move_lines(
-                    'out_customer'))
-        elif type_ == 'customer_supplier':
-            account_move_lines = self._get_account_stock_move_lines(
-                'in_customer')
-            account_move_lines.extend(self._get_account_stock_move_lines(
-                    'out_supplier'))
-        else:
-            account_move_lines = self._get_account_stock_move_lines(type_)
+        with Transaction().set_context(date=date):
+            if type_ == 'supplier_customer':
+                account_move_lines = self._get_account_stock_move_lines(
+                    'in_supplier')
+                account_move_lines.extend(self._get_account_stock_move_lines(
+                        'out_customer'))
+            elif type_ == 'customer_supplier':
+                account_move_lines = self._get_account_stock_move_lines(
+                    'in_customer')
+                account_move_lines.extend(self._get_account_stock_move_lines(
+                        'out_supplier'))
+            else:
+                account_move_lines = self._get_account_stock_move_lines(type_)
 
         amount = Decimal('0.0')
         for line in account_move_lines:
